@@ -324,7 +324,26 @@ check_for_firmware() {
 	rm -f /tmp/pkginstalled
 }
 
+# For those who don't want to load any firmware, even if available on
+# installation images (#1029848). The loop is still entered so that
+# logs are generated.
+db_get hw-detect/firmware-lookup
+firmware_lookup="$RET"
+
+# NOTE: The ask_load_firmware function returns true the first time around,
+# without asking any questions. For consistency, skip mountmedia calls during
+# the first iteration of the loop. For systems which have all firmware material
+# found in {,/cdrom}/firmware, this also means a noticeable speed-up.
+loop=0
 while check_missing && ask_load_firmware; do
+	loop=$((loop+1))
+	log "mainloop iteration #$loop"
+
+	if [ "$firmware_lookup" = "never" ]; then
+		log "firmware lookup disabled (=$firmware_lookup), exiting"
+		exit 0
+	fi
+
 	# first, check if needed firmware debs are available on the
 	# PXE initrd or the installation CD.
 	if [ -d /firmware ]; then
@@ -334,6 +353,24 @@ while check_missing && ask_load_firmware; do
 		check_for_firmware /cdrom/firmware
 	fi
 
+	# Whether we should keep both mountmedia calls, and whether mountmedia
+	# is doing a good job is discussed in #1029543:
+	if [ "$loop" -lt 1 ]; then
+		# second, look for loose firmware files on the media device.
+		if mountmedia; then
+			for file in $files; do
+				try_copy "$file"
+			done
+			umount /media || true
+		fi
+
+		# last, look for firmware debs on the media device
+		if mountmedia driver; then
+			check_for_firmware /media /media/firmware
+			umount /media || true
+		fi
+	fi
+
 	# remove and reload modules so they see the new firmware
 	for module in $modules; do
 		if ! nic_is_configured $module; then
